unit mouse;

(* Non tutte le funzioni sono sicuramente presenti
   nei vari driver per i mouse quindi accertarsi
   che il vostro driver preveda le funzioni che
   volete utilizzare prima di scrivere un programma.
   Le routine sono scritte con un driver Z-NIX in
   versione 6.11 compatibile Microsoft *)

interface

uses dos;

type scrmask=array[0..1,0..15] of integer;
     senstype=1..100;
     scaletype=1..maxint;
     proc=procedure;

const key_left=0;
      key_right=1;
      key_middle=2;
      hardware=1;
      software=0;

var max_key:integer;
    mouse_exist:boolean;
    prog:proc;
    posx,posy:integer;
    left,right:boolean;
    oldmouseexitproc:pointer;
    mouse_visible:boolean;

function get_speed:integer;
function get_video_page:integer;
function istrue(a,b,c:boolean):boolean;
procedure reset_mouse;
procedure show_cursor;
procedure hide_cursor;
procedure get_mouse_position(var x,y:integer);
procedure get_mouse_key(var kl,kr,km:boolean);
procedure set_mouse_position(x,y:integer);
procedure get_key_pressed_information(key:integer;
                      var pressed:integer;
                      var lstx,lsty:integer);
procedure get_key_released_information(key:integer;
                        var pressed:integer;
                        var lstx,lsty:integer);
procedure set_limit(xmin,ymin,xmax,ymax:integer);
procedure set_graphic_cursor(offx,offy:integer;
                     var arr:scrmask);
procedure set_text_cursor(curs,scmask,crmask:integer);
procedure get_offset(var offsx,offsy:integer);
procedure set_speed(speed:integer);
procedure set_sensitivity(xsens,ysens:senstype);
procedure get_sensitivity(var xsens,ysens:senstype);
procedure set_video_page(page:integer);
procedure set_scale(scalex,scaley:scaletype);
procedure enable_user_procedure(mscan:integer;user:proc;len:word);
procedure disable_user_procedure;
implementation

var reg:registers;

function istxtmode:boolean;
(* controlla se il programma si
   trova in modo testo o grafico *)
begin
  reg.ah:=15;
  intr(16,reg);
  if reg.al in [0..3,7] then istxtmode:=true else istxtmode:=false;
end;

procedure disable_user_procedure;
(* Disabilita la routine utente *)
begin
  if mouse_exist then begin
    reg.ax:=12;
    reg.cx:=0;
    reg.dx:=0;
    reg.es:=0;
    intr($33,reg);
  end;
end;

(*$F+*)
procedure MouseExitProc;
(* Utilizzata per disinstallare la
   procedura utente all'uscita del
   programma *)
begin
  ExitProc:=OldmouseExitProc;
  disable_user_procedure;
end;
(*$F-*)

function istrue(a,b,c:boolean):boolean;
(* Ritorna vero se il primo parametro
    vero e gli altri sono falsi *)
begin
  if a and not b and not c then istrue:=true else istrue:=false;
end;

procedure reset_mouse;
(* Resetta lo stato del mouse *)
begin
  if mouse_exist then begin
    reg.ax:=0;
    intr($33,reg);
    mouse_visible:=false;
  end;
end;

Procedure show_cursor;
(* Ripristina la visualizzazione
   del puntatore del mouse *)
begin
  if mouse_exist then begin
    if not(mouse_visible) then begin
      reg.ax:=1;
      intr($33,reg);
      mouse_visible:=true;
    end;
  end;
end;

procedure hide_cursor;
(* Toglie la visualizzazione del
   puntatore del mouse *)
begin
  if mouse_exist then begin
    if mouse_visible then begin
      reg.ax:=2;
      intr($33,reg);
      mouse_visible:=false;
    end;
  end;
end;

procedure get_mouse_position(var x,y:integer);
(* Ritorna in X ed in Y le coordinate del mouse
   dividendole per 8 nei modi testo *)
begin
  if mouse_exist then begin
    reg.ax:=3;
    intr($33,reg);
    if istxtmode then begin
      reg.cx:=reg.cx div 8;
      reg.dx:=reg.dx div 8;
    end;
    x:=reg.cx;
    y:=reg.dx;
  end else begin
    x:=0;
    y:=0;
  end;
end;

procedure get_mouse_key(var kl,kr,km:boolean);
(* Ritorna lo stato dei tre tasti del mouse *)
begin
  if mouse_exist then begin
    reg.ax:=3;
    intr($33,reg);
    kl:=(reg.bx and 1)=1;
    kr:=(reg.bx and 2)=2;
    km:=(reg.bx and 4)=4;
  end else begin
    kl:=false;
    kr:=false;
    km:=false;
  end;
end;

procedure set_mouse_position(x,y:integer);
(* Setta la posizione del puntatore del mouse *)
begin
  if mouse_exist then begin
    reg.ax:=4;
    reg.cx:=x;
    reg.dx:=y;
    if istxtmode then begin
      reg.cx:=reg.cx * 8;
      reg.dx:=reg.dx * 8;
    end;
    intr($33,reg);
  end;
end;

procedure get_key_pressed_information(key:integer;
                                  var pressed:integer;
                                  var lstx,lsty:integer);
(* Ritorna le informazioni riguardanti
   lo stato del mouse all'ultima pressione
   del tasto KEY *)
begin
  if mouse_exist then begin
    if key<=max_key then begin
      reg.ax:=5;
      reg.bx:=key;
      intr($33,reg);
      pressed:=reg.bx;
      if istxtmode then begin
        reg.cx:=reg.cx div 8;
        reg.dx:=reg.dx div 8;
      end;
      lstx:=reg.cx;
      lsty:=reg.dx;
    end else begin
      pressed:=0;
      lstx:=0;
      lsty:=0;
    end;
  end else begin
    pressed:=0;
    lstx:=0;
    lsty:=0;
  end;
end;

procedure get_key_released_information(key:integer;
                                   var pressed:integer;
                                   var lstx,lsty:integer);
(* Ritorna le informazioni riguardanti
   lo stato del mouse al rilascio dopo
   l'ultima pressione del tasto KEY *)
begin
  if mouse_exist then begin
    if key<=max_key then begin
      reg.ax:=6;
      reg.bx:=key;
      intr($33,reg);
      pressed:=reg.bx;
      if istxtmode then begin
        reg.cx:=reg.cx div 8;
        reg.dx:=reg.dx div 8;
      end;
      lstx:=reg.cx;
      lsty:=reg.dx;
    end else begin
      pressed:=0;
      lstx:=0;
      lsty:=0;
    end;
  end else begin
    pressed:=0;
    lstx:=0;
    lsty:=0;
  end;
end;

procedure set_limit(xmin,ymin,xmax,ymax:integer);
(* Setta le coordinate in cui puo' essere
   utilizzato il puntatore del mouse *)
begin
  if mouse_exist then begin
    reg.ax:=7;
    reg.cx:=xmin;
    reg.dx:=xmax;
    if istxtmode then begin
      reg.cx:=reg.cx * 8;
      reg.dx:=reg.dx * 8;
    end;
    intr($33,reg);
    reg.ax:=8;
    reg.cx:=ymin;
    reg.dx:=ymax;
      if istxtmode then begin
        reg.cx:=reg.cx * 8;
        reg.dx:=reg.dx * 8;
      end;
    intr($33,reg);
  end;
end;

procedure set_graphic_cursor(offx,offy:integer;
                         var arr:scrmask);
(* Setta la forma del puntatore in modo grafico
   offx ed offy rappresentano gli offset del punto
   di selezione delle opzioni *)
begin
  if mouse_exist then begin
    reg.ax:=9;
    reg.bx:=offx;
    reg.cx:=offy;
    reg.es:=seg(arr[0,0]);
    reg.dx:=ofs(arr[0,0]);
    intr($33,reg);
  end;
end;

procedure set_text_cursor(curs,scmask,crmask:integer);
(* Setta la forma del puntatore (scmask, crmask)
   in modo testo e il tipo di cursore (curs) *)
begin
  if mouse_exist then begin
    reg.ax:=10;
    reg.bx:=curs;
    reg.cx:=scmask;
    reg.dx:=crmask;
    intr($33,reg);
  end;
end;

procedure get_offset(var offsx,offsy:integer);
(* Ritorna lo spostamento del mouse
   in centesimi di pollice *)
begin
  if mouse_exist then begin
    reg.ax:=11;
    intr($33,reg);
    offsx:=reg.cx;
    offsy:=reg.dx;
  end else begin
    offsx:=0;
    offsy:=0;
  end;
end;

procedure set_scale(scalex,scaley:scaletype);
(* Setta la scala utilizzata per gli spostamenti
   cioe' di quanti centesimi di pollice deve essere
   spostato fisicamente il mouse per muovere il
   puntatore sullo schermo di 8 pixel *)
begin
  if mouse_exist then begin
    reg.ax:=15;
    reg.cx:=scalex;
    reg.dx:=scaley;
    intr($33,reg);
  end;
end;

procedure set_speed(speed:integer);
(* Setta la velocita' di spostamento cioe'
   il numero di centesimi di pollice per secondo.
   Se si muove il mouse piu' velocemente il fattore
   di scala viene modificato per permettere una risposta
   migliore del mouse *)
begin
  if mouse_exist then begin
    reg.ax:=19;
    reg.dx:=speed;
    intr($33,reg);
  end;
end;

function get_speed:integer;
(* Ritorna la velocita' di spostamento *)
begin
  if mouse_exist then begin
    reg.ax:=27;
    intr($33,reg);
    get_speed:=reg.dx;
  end else get_speed:=0;
end;

procedure set_sensitivity(xsens,ysens:senstype);
(* Setta la sensibilita' del mouse *)
begin
  if mouse_exist then begin
    reg.ax:=26;
    reg.bx:=xsens;
    reg.cx:=ysens;
    reg.dx:=get_speed;
    intr($33,reg);
  end;
end;

procedure get_sensitivity(var xsens,ysens:senstype);
(* Ritorna la sensibilita' del mouse *)
begin
  if mouse_exist then begin
    reg.ax:=27;
    intr($33,reg);
    xsens:=reg.bx;
    ysens:=reg.cx;
  end else begin
    xsens:=1;
    ysens:=1;
  end;
end;

procedure set_video_page(page:integer);
(* Setta la pagina video utilizzata *)
begin
  if mouse_exist then begin
    reg.ax:=29;
    reg.bx:=page;
    intr($33,reg);
  end;
end;

function get_video_page:integer;
(* Ritorna la pagina video *)
begin
  if mouse_exist then begin
    reg.ax:=30;
    intr($33,reg);
    get_video_page:=reg.bx;
  end else get_video_page:=0;
end;

(*$F+*)
(*$L MOUSE*)
procedure mymouse;
external;

procedure MSinit(pr:proc;stklen:word);
external;
(*F-*)

procedure enable_user_procedure(mscan:integer;user:proc;len:word);
(* Setta la procedura definita dall'utente (proc)
   con uno stack di len byte che verra' richiamata
   quando si avvera la condizione MSCAN:
   MSCAN bit  Condizione
     0        Cambio posizione del cursore
     1        Pressione del tasto sinistro
     2        Rilascio del tasto sinistro
     3        Pressione del tasto destro
     4        Rilascio del tasto destro
     5        Pressione del tasto centrale
     6        Rilascio del tasto centrale
   7..15      Non utilizzati

   La routine utente avra' nelle variabili
   LEFT, RIGHT, MIDDLE lo stato del tasto
   e nelle variabili X ed Y le coordinate
   del mouse. *)
begin
  if mouse_exist then begin
    prog:=user;
    msinit(prog,len);
    reg.ax:=12;
    reg.cx:=mscan;
    reg.dx:=ofs(mymouse);
    reg.es:=seg(mymouse);
    intr($33,reg);
  end;
end;

(* Inizializzazione della unit MOUSE.
   Viene controllata la presenza del mouse
   e vengono settate alcune variabili
   di sistema *)
begin
  reg.ax:=0;
  intr($33,reg);
  if reg.ax=$ffff then begin
    max_key:=reg.bx;
    mouse_exist:=true;
    OldmouseExitProc := ExitProc;
    ExitProc := @MouseExitProc;
  end else begin
    mouse_exist:=false;
    max_key:=0;
  end;
  mouse_visible:=false;
end.
